#include "Arduino.h"
#include <STM32_CAN.h>
#include "flash.h"



STM32_CAN Can(CAN2, DEF);

volatile bool fw_update = false;
volatile uint32_t fw_size = 0;
volatile uint16_t fw_crc = 0;
volatile uint32_t jump;
static FLASH_RECORD *flash_record = {0};
static uint32_t ptr_flash;

volatile uint32_t msg_id;
volatile uint16_t id_x;
volatile uint8_t msg_ch;

// Прототипы функций
void jump_to_app();
void process_can_message(const CAN_message_t &msg);
void erase_flash_pages();
bool verify_firmware();
void send_ack(uint8_t status);

void setup() {
    // Инициализация периферии
    SystemInit();
    SystemClock_Config();
    Serial.setRx(HARDWARE_SERIAL_RX_PIN);
    Serial.setTx(HARDWARE_SERIAL_TX_PIN);
    Serial.begin(115200);
    Can.begin();
    Can.setBaudRate(1000000);
    TIM_TypeDef *Instance = TIM2;
    HardwareTimer *SendTimer = new HardwareTimer(Instance);
    SendTimer->setOverflow(100, HERTZ_FORMAT);  // 50 Hz
   // SendTimer->attachInterrupt(process_can_message);
    SendTimer->resume();
    // Разрешить все ID (маска 0x00000000)
    Can.setFilter(0, 0, STD);

    // Настройка GPIO
    RCC->AHB1ENR |= RCC_AHB1ENR_GPIOCEN;
    GPIOC->MODER |= GPIO_MODER_MODE10_0 | GPIO_MODER_MODE11_0;
    GPIOC->ODR &= ~GPIO_ODR_OD11;
    GPIOC->ODR |= GPIO_ODR_OD10;
    // // Проверка флага обновления
    // erase_sector(7);
    // flash_program_word(FLAG_BOOT,0xDEADBEEF,BYTE32);
    // flash_record = load_params();
    // write_param(firmw,UPDATE_FLAG);
    flash_record = load_params();
    // write_param(addr_id,33);
    // flash_record = load_params();
   /*           Добавить проверку адреса, т.е во время отправки запроса прошивки по CAN
                мы сохраняем как флаг, так и аддрес устройства к которому будет обращатлься во 
                время прошивки                                      */

    if(flash_record[firmw].value == UPDATE_FLAG) {
        fw_update = true;
        for(int i = 0; i < 5;i++){
            GPIOC->ODR ^= GPIO_ODR_OD10;  // Индикация обновления
            delay(100);
        }
        erase_flash_pages();
    } else {
        jump_to_app();
    }
    GPIOC->ODR |= GPIO_ODR_OD10;
}



void process_can_message(const CAN_message_t &msg) {
     msg_id = msg.id;
    /*          0x691  
    69  -   адрес устройства    
    1   -   что делать дальше с данными     */

    id_x = (msg_id >> 4) & 0xFFFF;  //получение адреса устройства страшие 2 бита
    msg_ch  = msg_id & 0xF;         // получения id, чтобы выбрать, что делать
    if(id_x == flash_record[addr_id].value){
    switch(msg_ch) {
        case BOOT_CAN_ID:
            if(msg.buf[0] == 0x01) {  // Старт передачи
                fw_size = *(uint32_t*)&msg.buf[1];  //размер прошивки тип 4 байта
                fw_crc = *(uint16_t*)&msg.buf[5];  //crc
                ptr_flash = APP_ADDRESS;
                send_ack(0x01);
            }
            break;

        case DATA_CAN_ID:  // Пакет данных
            if(ptr_flash < (APP_ADDRESS + fw_size)) {
                write_flash_page((const uint8_t*)msg.buf, msg.len);
                ptr_flash += msg.len;
                send_ack(0x02);

            }
            break;
        

        case BOOT_CAN_END:  // Завершение передачи
            if(verify_firmware()) {
                send_ack(0xAA);
                HAL_Delay(500);
                // erase_sector(7);  // Сброс флага
                write_param(firmw,0);   //reset flasg
                NVIC_SystemReset();
            } else {
                send_ack(0x55);
                erase_flash_pages();    //if error
            }
            break;
    }
}
}

void jump_to_app() {
    __disable_irq();
    jump = *(volatile uint32_t*)(APP_ADDRESS + 4);
    void (*app_entry)(void);
    app_entry = (void (*)(void))jump;


    for (uint32_t i = 0; i < 8; i++) {
        NVIC->ICPR[i] = 0xFFFFFFFF;
    }

   __set_MSP(*(volatile uint32_t*)APP_ADDRESS);
   	// SCB->VTOR = (uint32_t)0x08008004;
    app_entry();
}





volatile uint32_t add = (uint32_t)APP_ADDRESS;


bool verify_firmware() {
    uint16_t calculated_crc = 0;
    calculated_crc = validate_crc16((uint8_t*)add,fw_size);
    return (calculated_crc == fw_crc);
}

void send_ack(uint8_t status) {

    CAN_message_t ack;
    ack.id = ACK_CAN_ID;
    ack.len = 1;
    ack.buf[0] = status;
    Can.write(ack);
}


void loop() {
    if(fw_update) {
    CAN_message_t msg;
    while(Can.read(msg)) 
        process_can_message(msg);
    }
}